@@ -2,7 +2,7 @@
 """
 Created on Wed Dec 23 20:37:52 2020
 
-:copyright: 
+:copyright:
     Jared Peacock (jpeacock@usgs.gov)
 
 :license: MIT
@@ -46,15 +46,36 @@ def wrap_description(description, column_width):
     d_lines = textwrap.wrap(description, column_width)
     if len(d_lines) < 11:
         d_lines += [""] * (11 - len(d_lines))
-
     return d_lines
 
 
+def validate_c1(attr_dict, c1):
+    """
+
+    :param attr_dict: DESCRIPTION
+    :type attr_dict: TYPE
+    :param c1: DESCRIPTION
+    :type c1: TYPE
+    :return: DESCRIPTION
+    :rtype: TYPE
+
+    """
+    try:
+        max_c1 = max([len(key) for key in attr_dict.keys()])
+
+        if max_c1 > (c1 - 4):
+            c1 = max_c1 + 6
+    except ValueError:
+        pass
+
+    return c1
+
+
 def write_lines(attr_dict, c1=45, c2=45, c3=15):
     """
     Takes the attribute dictionary from the json and parses it into a table
     Returns a string representation of this table.  This overwrites the doc.
-    
+
     :param attr_dict: attribute dictionary
     :type attr_dict: dict
     :param c1: cloumn 1 width, defaults to 45
@@ -67,6 +88,7 @@ def write_lines(attr_dict, c1=45, c2=45, c3=15):
     :rtype: string
 
     """
+    c1 = validate_c1(attr_dict, c1)
 
     line = "       | {0:<{1}}| {2:<{3}} | {4:<{5}}|"
     hline = "       +{0}+{1}+{2}+".format(
@@ -78,7 +100,9 @@ def write_lines(attr_dict, c1=45, c2=45, c3=15):
 
     lines = [
         hline,
-        line.format("**Metadata Key**", c1, "**Description**", c2, "**Example**", c3),
+        line.format(
+            "**Metadata Key**", c1, "**Description**", c2, "**Example**", c3
+        ),
         mline,
     ]
 
@@ -88,13 +112,20 @@ def write_lines(attr_dict, c1=45, c2=45, c3=15):
         d_lines = wrap_description(entry["description"], c2)
         e_lines = wrap_description(entry["example"], c3)
         # line 1 is with the entry
-        lines.append(line.format(f"**{key}**", c1, d_lines[0], c2, e_lines[0], c3))
+        lines.append(
+            line.format(f"**{key}**", c1, d_lines[0], c2, e_lines[0], c3)
+        )
         # line 2 skip an entry in the
         lines.append(line.format("", c1, d_lines[1], c2, e_lines[1], c3))
         # line 3 required
         lines.append(
             line.format(
-                f"Required: {entry['required']}", c1, d_lines[2], c2, e_lines[2], c3
+                f"Required: {entry['required']}",
+                c1,
+                d_lines[2],
+                c2,
+                e_lines[2],
+                c3,
             )
         )
         # line 4 blank
@@ -102,7 +133,9 @@ def write_lines(attr_dict, c1=45, c2=45, c3=15):
 
         # line 5 units
         lines.append(
-            line.format(f"Units: {entry['units']}", c1, d_lines[4], c2, e_lines[4], c3)
+            line.format(
+                f"Units: {entry['units']}", c1, d_lines[4], c2, e_lines[4], c3
+            )
         )
 
         # line 6 blank
@@ -110,7 +143,9 @@ def write_lines(attr_dict, c1=45, c2=45, c3=15):
 
         # line 7 type
         lines.append(
-            line.format(f"Type: {entry['type']}", c1, d_lines[6], c2, e_lines[6], c3)
+            line.format(
+                f"Type: {entry['type']}", c1, d_lines[6], c2, e_lines[6], c3
+            )
         )
 
         # line 8 blank
@@ -118,31 +153,68 @@ def write_lines(attr_dict, c1=45, c2=45, c3=15):
 
         # line 9 type
         lines.append(
-            line.format(f"Style: {entry['style']}", c1, d_lines[8], c2, e_lines[8], c3)
+            line.format(
+                f"Style: {entry['style']}", c1, d_lines[8], c2, e_lines[8], c3
+            )
         )
-        
+
         # line 10 blank
         lines.append(line.format("", c1, d_lines[9], c2, e_lines[9], c3))
-        
-        # line 11 type
+
+        default = [entry["default"]] + [""] * 5
+        if len(str(entry["default"])) > c1 - 15:
+            default = [""] + wrap_description(entry["default"], c1)
+
+        # line 9 type
         lines.append(
-            line.format(f"Default: {entry['default']}", c1, d_lines[10], c2, e_lines[10], c3)
+            line.format(
+                f"**Default**: {default[0]}",
+                c1,
+                d_lines[8],
+                c2,
+                e_lines[8],
+                c3,
+            )
+        )
+
+        # line 10 blank
+        lines.append(
+            line.format(default[1], c1, d_lines[9], c2, e_lines[9], c3)
+        )
+
+        # line 9 type
+        lines.append(
+            line.format(default[2], c1, d_lines[10], c2, e_lines[10], c3)
         )
 
         # line 10 blank
         if len(d_lines) > 11:
-            lines.append(line.format("", c1, d_lines[9], c2, "", c3))
-            for d_line in d_lines[10:]:
-                lines.append(line.format("", c1, d_line, c2, "", c3))
+            lines.append(line.format(default[3], c1, d_lines[11], c2, "", c3))
+            for index, d_line in enumerate(d_lines[12:], 4):
+                try:
+                    lines.append(
+                        line.format(default[index], c1, d_line, c2, "", c3)
+                    )
+                except IndexError:
+                    lines.append(line.format("", c1, d_line, c2, "", c3))
 
+        # long default value
+        if len(default) > 7:
+            lines.append(line.format(default[3], c1, "", c2, "", c3))
+            for index, d_line in enumerate(default[4:], 12):
+                try:
+                    lines.append(
+                        line.format(d_line, c1, d_lines[index], c2, "", c3)
+                    )
+                except IndexError:
+                    lines.append(line.format(d_line, c1, "", c2, "", c3))
         lines.append(hline)
-
     return "\n".join(lines)
 
 
 def write_block(key, attr_dict, c1=45, c2=45, c3=15):
     """
-    
+
     :param key: key to write from attr dict
     :type key: string
     :param attr_dict: attribute dictionary
@@ -157,6 +229,8 @@ def write_block(key, attr_dict, c1=45, c2=45, c3=15):
     :rtype: list
 
     """
+    if len(key) > c1 - 4:
+        c1 = len(key) + 6
 
     line = "       | {0:<{1}}| {2:<{3}} | {4:<{5}}|"
     hline = "       +{0}+{1}+{2}+".format(
@@ -187,7 +261,12 @@ def write_block(key, attr_dict, c1=45, c2=45, c3=15):
     # line 1 is with the entry
     lines.append(
         line.format(
-            f"**Required**: {attr_dict['required']}", c1, d_lines[0], c2, e_lines[0], c3
+            f"**Required**: {attr_dict['required']}",
+            c1,
+            d_lines[0],
+            c2,
+            e_lines[0],
+            c3,
         )
     )
     # line 2 skip an entry in the
@@ -195,7 +274,12 @@ def write_block(key, attr_dict, c1=45, c2=45, c3=15):
     # line 3 required
     lines.append(
         line.format(
-            f"**Units**: {attr_dict['units']}", c1, d_lines[2], c2, e_lines[2], c3
+            f"**Units**: {attr_dict['units']}",
+            c1,
+            d_lines[2],
+            c2,
+            e_lines[2],
+            c3,
         )
     )
     # line 4 blank
@@ -204,7 +288,12 @@ def write_block(key, attr_dict, c1=45, c2=45, c3=15):
     # line 5 units
     lines.append(
         line.format(
-            f"**Type**: {attr_dict['type']}", c1, d_lines[4], c2, e_lines[4], c3
+            f"**Type**: {attr_dict['type']}",
+            c1,
+            d_lines[4],
+            c2,
+            e_lines[4],
+            c3,
         )
     )
 
@@ -214,31 +303,61 @@ def write_block(key, attr_dict, c1=45, c2=45, c3=15):
     # line 7 type
     lines.append(
         line.format(
-            f"**Style**: {attr_dict['style']}", c1, d_lines[6], c2, e_lines[6], c3
+            f"**Style**: {attr_dict['style']}",
+            c1,
+            d_lines[6],
+            c2,
+            e_lines[6],
+            c3,
         )
     )
 
     # line 8 blank
     lines.append(line.format("", c1, d_lines[7], c2, e_lines[7], c3))
-    
+
+    default = [attr_dict["default"]] + [""] * 5
+    if len(str(attr_dict["default"])) > c1 - 15:
+        default = [""] + wrap_description(attr_dict["default"], c1)
+
     # line 9 type
     lines.append(
         line.format(
-            f"**Default**: {attr_dict['default']}", c1, d_lines[8], c2, e_lines[8], c3
+            f"**Default**: {default[0]}",
+            c1,
+            d_lines[8],
+            c2,
+            e_lines[8],
+            c3,
         )
     )
 
     # line 10 blank
-    lines.append(line.format("", c1, d_lines[9], c2, e_lines[9], c3))
+    lines.append(line.format(default[1], c1, d_lines[9], c2, e_lines[9], c3))
 
     # line 9 type
-    lines.append(line.format("", c1, d_lines[10], c2, e_lines[10], c3))
+    lines.append(line.format(default[2], c1, d_lines[10], c2, e_lines[10], c3))
 
     # line 10 blank
     if len(d_lines) > 11:
-        lines.append(line.format("", c1, d_lines[11], c2, "", c3))
-        for d_line in d_lines[12:]:
-            lines.append(line.format("", c1, d_line, c2, "", c3))
+        lines.append(line.format(default[3], c1, d_lines[11], c2, "", c3))
+        for index, d_line in enumerate(d_lines[12:], 4):
+            try:
+                lines.append(
+                    line.format(default[index], c1, d_line, c2, "", c3)
+                )
+            except IndexError:
+                lines.append(line.format("", c1, d_line, c2, "", c3))
+
+    # long default value
+    if len(default) > 7:
+        lines.append(line.format(default[3], c1, "", c2, "", c3))
+        for index, d_line in enumerate(default[4:], 12):
+            try:
+                lines.append(
+                    line.format(d_line, c1, d_lines[index], c2, "", c3)
+                )
+            except IndexError:
+                lines.append(line.format(d_line, c1, "", c2, "", c3))
 
     lines.append(hline)
     lines.append("")
@@ -264,10 +383,9 @@ def flatten_dict(meta_dict, parent_key=None, sep="."):
     items = []
     for key, value in meta_dict.items():
         if parent_key:
-            new_key = "{0}{1}{2}".format(parent_key, sep, key)
+            new_key = f"{parent_key}{sep}{key}"
         else:
             new_key = key
-
         if isinstance(value, MutableMapping):
             items.extend(flatten_dict(value, new_key, sep=sep).items())
         else:
@@ -318,20 +436,54 @@ def recursive_split_getattr(base_object, name, sep="."):
 
     if other:
         base_object = getattr(base_object, key)
-        value = recursive_split_getattr(base_object, other[0])
+        value, prop = recursive_split_getattr(base_object, other[0])
     else:
         value = getattr(base_object, key)
-    return value
+        try:
+            if isinstance(getattr(type(base_object), key), property):
+                prop = True
+        except AttributeError:
+            prop = False
+    return value, prop
 
 
-def recursive_split_setattr(base_object, name, value, sep="."):
+def recursive_split_setattr(
+    base_object, name, value, sep=".", skip_validation=False
+):
+    """
+    Recursively split a name and set the value of the last key. Recursion splits on the separator present in the name.
+
+    :param base_object: The object having its attribute set, or a "parent" object in the recursive/nested scenario
+    :type base_object: object
+    :param name: The name of the attribute to set
+    :type name: str
+    :param value: The value to set the attribute to
+    :type value: any
+    :param sep: The separator to split the name on, defaults to "."
+    :type sep: str, optional
+    :param skip_validation: Whether to skip validation/parse of the attribute, defaults to False
+    :type skip_validation: Optional[bool]
+
+    :return: None
+    :rtype: NoneType
+
+    """
     key, *other = name.split(sep, 1)
 
-    if other:
-        base_object = getattr(base_object, key)
-        recursive_split_setattr(base_object, other[0], value)
+    if skip_validation:
+        if other:
+            base_object = getattr(base_object, key)
+            recursive_split_setattr(
+                base_object, other[0], value, skip_validation=True
+            )
+        else:
+            base_object.setattr_skip_validation(key, value)
     else:
-        setattr(base_object, key, value)
+        if other:
+            base_object = getattr(base_object, key)
+            recursive_split_setattr(base_object, other[0], value)
+        else:
+            setattr(base_object, key, value)
 
 
 def structure_dict(meta_dict, sep="."):
@@ -361,7 +513,6 @@ def get_units(name, attr_dict):
         units = None
     if units in [None, "None", "none"]:
         return None
-
     return units
 
 
@@ -385,12 +536,10 @@ def recursive_split_xml(element, item, base, name, attr_dict=None):
 
             sub_element = et.SubElement(element, key)
             recursive_split_xml(sub_element, value, attr_name, key, attr_dict)
-
     elif isinstance(item, (tuple, list)):
         for ii in item:
             sub_element = et.SubElement(element, "item")
             recursive_split_xml(sub_element, ii, base, name, attr_dict)
-
     elif isinstance(item, str):
         element.text = item
     elif isinstance(item, (float, int, type(None))):
@@ -401,17 +550,14 @@ def recursive_split_xml(element, item, base, name, attr_dict=None):
             element.text = str(item)
         else:
             raise ValueError("Value cannot be {0}".format(type(item)))
-
     if attr_dict:
 
         units = get_units(base, attr_dict)
         if units:
             element.set("units", str(units))
-
         # v_type = get_type(base, attr_dict)
         # if v_type:
         #     element.set("type", v_type)
-
     return element, name
 
 
@@ -431,7 +577,6 @@ def dict_to_xml(meta_dict, attr_dict=None):
     for key, value in meta_dict[class_name].items():
         element = et.SubElement(root, key)
         recursive_split_xml(element, value, key, key, attr_dict)
-
     return root
 
 
@@ -454,11 +599,12 @@ def element_to_dict(element):
             for k, v in dc.items():
                 child_dict[k].append(v)
         meta_dict = {
-            element.tag: {k: v[0] if len(v) == 1 else v for k, v in child_dict.items()}
+            element.tag: {
+                k: v[0] if len(v) == 1 else v for k, v in child_dict.items()
+            }
         }
         if "item" in meta_dict[element.tag].keys():
             meta_dict[element.tag] = meta_dict[element.tag]["item"]
-
     # going to skip attributes for now, later can check them against
     # standards, neet to skip units and type
     if element.attrib:
@@ -472,16 +618,24 @@ def element_to_dict(element):
                     pop_units = True
                     continue
             if k in ["type"]:
-                if len(element.attrib.keys()) <= 1:
-                    pop_type = True
-                    continue
-            meta_dict[element.tag][k] = v
 
+                if len(element.attrib.keys()) <= 1:
+                    if v in [
+                        "float",
+                        "string",
+                        "integer",
+                        "boolean",
+                        "list",
+                        "tuple",
+                    ]:
+                        pop_type = True
+                        continue
+
+            meta_dict[element.tag][k] = v
         if pop_units:
             element.attrib.pop("units")
         if pop_type:
             element.attrib.pop("type")
-
     if element.text:
         text = element.text.strip()
         if children or element.attrib:
@@ -492,12 +646,18 @@ def element_to_dict(element):
                     meta_dict[element.tag] = text
         else:
             meta_dict[element.tag] = text
-
     return OrderedDict(sorted(meta_dict.items(), key=itemgetter(0)))
 
 
 def element_to_string(element):
-    return minidom.parseString(et.tostring(element).decode()).toprettyxml(indent="    ")
+    return (
+        minidom.parseString(et.tostring(element).decode())
+        .toprettyxml(
+            indent="    ",
+            encoding="UTF-8",
+        )
+        .decode()
+    )
 
 
 # =============================================================================
@@ -509,6 +669,12 @@ class NumpyEncoder(json.JSONEncoder):
     """
 
     def default(self, obj):
+        """
+
+        :param obj:
+        :type obj:
+        :return:
+        """
         if isinstance(
             obj,
             (
@@ -526,15 +692,80 @@ class NumpyEncoder(json.JSONEncoder):
             ),
         ):
             return int(obj)
-
-        elif isinstance(obj, (np.float_, np.float16, np.float32, np.float64)):
+        elif isinstance(obj, (np.float16, np.float32, np.float64)):
             return float(obj)
-
         elif isinstance(obj, (np.ndarray)):
-            return obj.tolist()
-
+            if obj.dtype == complex:
+                return {"real": obj.real.tolist(), "imag": obj.imag.tolist()}
+            else:
+                return obj.tolist()
         # For now turn references into a generic string
         elif "h5" in str(type(obj)):
             return str(obj)
-
         return json.JSONEncoder.default(self, obj)
+
+
+def validate_name(name, pattern=None):
+    """
+    Validate name
+
+    :param name: DESCRIPTION
+    :type name: TYPE
+    :param pattern: DESCRIPTION, defaults to None
+    :type pattern: TYPE, optional
+    :return: DESCRIPTION
+    :rtype: TYPE
+
+    """
+    if name is None:
+        return "unknown"
+    return name.replace(" ", "_")
+
+
+def requires(**requirements):
+    """Decorate a function with optional dependencies.
+
+    Parameters
+    ----------
+    **requirements : obj
+        keywords of package name and the required object for
+        a function.
+
+    Returns
+    -------
+    decorated_function : function
+        Original function if all soft dependencies are met, otherwise
+        it returns an empty function which prints why it is not running.
+
+    Examples
+    --------
+    ```
+    try:
+        import obspy
+    except ImportError:
+        obspy = None
+
+    @requires(obspy=obspy)
+    def obspy_function():
+        ...
+        # does something using obspy
+
+    """
+    # Check the requirements, add missing package name in the list `missing`.
+    missing = []
+    for key, item in requirements.items():
+        if not item:
+            missing.append(key)
+
+    def decorated_function(function):
+        """Wrap function."""
+        if not missing:
+            return function
+        else:
+            def passer(*args, **kwargs):
+                print(("Missing dependencies: {d}.".format(d=missing)))
+                print(("Not running `{}`.".format(function.__name__)))
+
+            return passer
+
+    return decorated_function
