@@ -20,7 +20,7 @@ attr_dict = get_schema("instrument", SCHEMA_FN_PATHS)
 attr_dict.add_dict(
     get_schema("location", SCHEMA_FN_PATHS),
     None,
-    keys=["latitude", "longitude", "elevation"],
+    keys=["latitude", "longitude", "elevation", "x", "y", "z", "x2", "y2", "z2"],
 )
 
 # =============================================================================
@@ -28,12 +28,4 @@ class Electrode(Base):
     __doc__ = write_lines(attr_dict)
 
     def __init__(self, **kwargs):
-
-        self.id = None
-        self.manufacturer = None
-        self.type = None
-        self.model = None
-        self.latitude = 0.0
-        self.longitude = 0.0
-        self.elevation = 0.0
         super().__init__(attr_dict=attr_dict, **kwargs)
