@@ -22,11 +22,5 @@ class Fdsn(Base):
     __doc__ = write_lines(attr_dict)
 
     def __init__(self, **kwargs):
-        self.id = None
-        self.network = None
-        self.channel_code = None
-        self.new_epoch = None
-        self.alternate_code = None
-        self.alternate_network_code = None
 
         super().__init__(attr_dict=attr_dict, **kwargs)
