@@ -3,7 +3,7 @@ from .fir_filter import FIRFilter
 from .pole_zero_filter import PoleZeroFilter
 from .time_delay_filter import TimeDelayFilter
 from .frequency_response_table_filter import FrequencyResponseTableFilter
-from .channel_response_filter import ChannelResponseFilter
+from .channel_response import ChannelResponse
 
 
 __all__ = [
@@ -12,5 +12,5 @@ __all__ = [
     "PoleZeroFilter",
     "TimeDelayFilter",
     "FrequencyResponseTableFilter",
-    "ChannelResponseFilter",
+    "ChannelResponse",
 ]
