@@ -23,10 +23,4 @@ class Instrument(Base):
 
     def __init__(self, **kwargs):
 
-        self.id = None
-        self.manufacturer = None
-        self.type = None
-        self.model = None
-        self.name = None
-        self.settings = None
         super().__init__(attr_dict=attr_dict, **kwargs)
