@@ -17,22 +17,25 @@ from .standards import SCHEMA_FN_PATHS
 
 # =============================================================================
 attr_dict = get_schema("person", SCHEMA_FN_PATHS)
+
+
 # =============================================================================
 class Person(Base):
     __doc__ = write_lines(attr_dict)
 
     def __init__(self, **kwargs):
 
-        self.email = None
-        self.name = None
-        self.organization = None
-        self.comments = None
         super().__init__(attr_dict=attr_dict, **kwargs)
 
-    @property
-    def author(self):
-        return self.name
+    # @property
+    # def author(self):
+    #     if self.name is None:
+    #         return
+    #     return self.name
 
-    @author.setter
-    def author(self, value):
-        self.name = value
+    # @author.setter
+    # def author(self, value):
+    #     if value is None:
+    #         self.name = None
+    #     else:
+    #         self.name = value
