@@ -2,12 +2,14 @@
 """
 Created on Fri Feb 19 16:14:41 2021
 
-:copyright: 
+:copyright:
     Jared Peacock (jpeacock@usgs.gov)
 
 :license: MIT
 
 """
+import copy
+
 # =============================================================================
 # Imports
 # =============================================================================
@@ -24,6 +26,7 @@ from mt_metadata.timeseries.stationxml.utils import BaseTranslator
 from mt_metadata.utils.units import get_unit_object
 
 from obspy.core import inventory
+from obspy import UTCDateTime
 
 # =============================================================================
 
@@ -33,6 +36,15 @@ class XMLChannelMTChannel(BaseTranslator):
     translate back and forth between StationXML Channel and MT Channel
     """
 
+    understood_sensor_types = [
+        "logger",
+        "magnetometer",
+        "induction coil",
+        "coil",
+        "dipole",
+        "electrode",
+    ]
+
     def __init__(self):
         super().__init__()
 
@@ -66,7 +78,7 @@ class XMLChannelMTChannel(BaseTranslator):
         self.mt_comments_list = ["run.id"]
         self.run_list = None
 
-    def xml_to_mt(self, xml_channel):
+    def xml_to_mt(self, xml_channel, existing_filters={}):
         """
         Translate :class:`obspy.core.inventory.Channel` to
         :class:`mt_metadata.timeseries.Channel`
@@ -95,7 +107,7 @@ class XMLChannelMTChannel(BaseTranslator):
         mt_channel = self._parse_xml_comments(xml_channel.comments, mt_channel)
         mt_channel = self._sensor_to_mt(xml_channel.sensor, mt_channel)
         mt_channel = self._get_mt_units(xml_channel, mt_channel)
-        mt_filters = self._xml_response_to_mt(xml_channel)
+        mt_filters = self._xml_response_to_mt(xml_channel, existing_filters)
 
         for xml_key, mt_key in self.xml_translator.items():
             if mt_key:
@@ -108,8 +120,11 @@ class XMLChannelMTChannel(BaseTranslator):
 
         # fill channel filters
         mt_channel.filter.name = list(mt_filters.keys())
-        mt_channel.filter.applied = [False] * len(list(mt_filters.keys()))
-
+        mt_channel.filter.applied = [True] * len(list(mt_filters.keys()))
+        if UTCDateTime(mt_channel.time_period.end) < UTCDateTime(
+            mt_channel.time_period.start
+        ):
+            mt_channel.time_period.end = "2200-01-01T00:00:00+00:00"
         return mt_channel, mt_filters
 
     def mt_to_xml(self, mt_channel, filters_dict, hard_code=True):
@@ -126,7 +141,8 @@ class XMLChannelMTChannel(BaseTranslator):
         """
 
         if not isinstance(
-            mt_channel, (metadata.Electric, metadata.Magnetic, metadata.Auxiliary)
+            mt_channel,
+            (metadata.Electric, metadata.Magnetic, metadata.Auxiliary),
         ):
             msg = f"Input must be mt_metadata.timeseries.Channel object not {type(mt_channel)}"
             self.logger.error(msg)
@@ -210,6 +226,8 @@ class XMLChannelMTChannel(BaseTranslator):
         :rtype: TYPE
 
         """
+        sensor.type = self._deduce_sensor_type(sensor)
+
         if not sensor.type:
             return mt_channel
 
@@ -410,7 +428,10 @@ class XMLChannelMTChannel(BaseTranslator):
         :rtype: TYPE
 
         """
+
         comments = []
+        if mt_comment is None:
+            return comments
         clist = mt_comment.split("run_ids:")
         for item in clist:
             if ":" in item:
@@ -470,31 +491,35 @@ class XMLChannelMTChannel(BaseTranslator):
 
         return mt_channel
 
-    def _xml_response_to_mt(self, xml_channel):
+    def _xml_response_to_mt(self, xml_channel, existing_filters={}):
         """
         parse the filters from obspy into mt filters
         """
-        filter_dict = {}
+        ch_filter_dict = {}
         for i_stage, stage in enumerate(xml_channel.response.response_stages):
+            new_and_unnamed = False
             mt_filter = create_filter_from_stage(stage)
             if not mt_filter.name:
-                filter_number = self._add_filter_number(
-                    filter_dict.keys(), mt_filter.type
-                )
-                mt_filter.name = f"{mt_filter.type}_{filter_number:02}"
-
-                self.logger.info(
-                    f"Found an nnnamed filter, named it: '{mt_filter.name}'"
+                filter_name, new_and_unnamed = self._add_filter_number(
+                    existing_filters, mt_filter
                 )
+                mt_filter.name = filter_name
 
             if mt_filter.decimation_active:
                 # keep filter names unique if same one used more than once
                 mt_filter.name += f"_{mt_filter.decimation_input_sample_rate}"
-            filter_dict[mt_filter.name.lower()] = mt_filter
 
-        return filter_dict
+            if new_and_unnamed:
+                self.logger.info(
+                    f"Found an unnamed filter, named it: '{mt_filter.name}'"
+                )
+                existing_filters[filter_name] = mt_filter
 
-    def _add_filter_number(self, keys, filter_type):
+            ch_filter_dict[mt_filter.name.lower()] = mt_filter
+
+        return ch_filter_dict
+
+    def _add_filter_number(self, existing_filters, mt_filter):
         """
         return the next number the number of filters
 
@@ -504,14 +529,26 @@ class XMLChannelMTChannel(BaseTranslator):
         :rtype: TYPE
 
         """
+
+        # check for existing filters
+        for f_key, f_obj in existing_filters.items():
+            if f_obj.type == mt_filter.type:
+                if round(abs(f_obj.complex_response([1])[0])) == round(
+                    abs(mt_filter.complex_response([1])[0])
+                ):
+
+                    return f_obj.name, False
+
         try:
-            last = sorted([k for k in keys if filter_type in k])[-1]
+            last = sorted([k for k in existing_filters.keys() if mt_filter.type in k])[
+                -1
+            ]
         except IndexError:
-            return 0
+            return f"{mt_filter.type}_{0:02}", True
         try:
-            return int(last[-2:]) + 1
+            return f"{mt_filter.type}_{int(last[-2:]) + 1:02}", True
         except ValueError:
-            return 0
+            return f"{mt_filter.type}_{0:02}", True
 
     def _mt_to_xml_response(self, mt_channel, filters_dict, xml_channel):
         """
@@ -539,3 +576,56 @@ class XMLChannelMTChannel(BaseTranslator):
         xml_channel.calibration_units_description = unit_obj.name
 
         return xml_channel
+
+    def _deduce_sensor_type(self, sensor):
+        """
+
+        :param sensor: Information about a sensor, usually extractes from FDSN XML
+        :type sensor: obspy.core.inventory.util.Equipment
+
+        :return:
+        """
+        original_sensor_type = sensor.type
+        original_sensor_description = sensor.description
+        # set sensor_type to be a string if it is None
+        if original_sensor_type is None:
+            sensor_type = ""  # make a string
+            msg = f"Sensor {sensor} does not have field type attr"
+            self.logger.debug(msg)
+        else:
+            sensor_type = copy.deepcopy(original_sensor_type)
+
+        if original_sensor_description is None:
+            sensor_description = ""  # make a string
+        else:
+            sensor_description = copy.deepcopy(original_sensor_type)
+
+        if sensor_type.lower() in self.understood_sensor_types:
+            return sensor_type
+        else:
+            self.logger.warning(
+                f" sensor {sensor} type {sensor.type} not in {self.understood_sensor_types}"
+            )
+
+        #  Try handling Bartington FGM at Earthscope ... this is a place holder for handling non-standard cases
+        if sensor_type.lower() == "bartington":
+            sensor_type = "magnetometer"
+        if not sensor_type:
+            if sensor_description == "Bartington 3-Axis Fluxgate Sensor":
+                sensor_type = "magnetometer"
+            if sensor_description:
+                if ("bf-4" in sensor_description.lower()) & (
+                    "schlumberger" in sensor_description.lower()
+                ):  # BSL-NCEDC
+                    sensor_type = "magnetometer"
+                elif ("electric" in sensor_description.lower()) & (
+                    "dipole" in sensor_description.lower()
+                ):  # BSL-NCEDC
+                    sensor_type = "dipole"
+
+        # reset sensor_type to None it it was not handled
+        if not sensor_type:
+            sensor_type = original_sensor_type
+            self.logger.error("sensor type could not be resolved")
+
+        return sensor_type
