@@ -14,6 +14,7 @@ Created on Wed Dec 23 21:30:36 2020
 from mt_metadata.base.helpers import write_lines
 from mt_metadata.base import get_schema, Base
 from .standards import SCHEMA_FN_PATHS
+from mt_metadata.transfer_functions.io.emtfxml.metadata import helpers
 
 # =============================================================================
 attr_dict = get_schema("citation", SCHEMA_FN_PATHS)
@@ -22,11 +23,29 @@ class Citation(Base):
     __doc__ = write_lines(attr_dict)
 
     def __init__(self, **kwargs):
-        self.author = None
-        self.title = None
-        self.journal = None
-        self.volume = None
-        self.doi = None
-        self.year = None
-        self.survey_d_o_i = None
+
         super().__init__(attr_dict=attr_dict, **kwargs)
+
+    def to_xml(self, string=False, required=True):
+        """
+
+        :param string: DESCRIPTION, defaults to False
+        :type string: TYPE, optional
+        :param required: DESCRIPTION, defaults to True
+        :type required: TYPE, optional
+        :return: DESCRIPTION
+        :rtype: TYPE
+
+        """
+
+        return helpers.to_xml(
+            self,
+            string=string,
+            required=required,
+            order=[
+                "title",
+                "authors",
+                "year",
+                "survey_d_o_i",
+            ],
+        )
