@@ -14,6 +14,7 @@ Created on Wed Dec 23 21:30:36 2020
 from mt_metadata.base.helpers import write_lines
 from mt_metadata.base import get_schema, Base
 from .standards import SCHEMA_FN_PATHS
+from mt_metadata.transfer_functions.io.emtfxml.metadata import helpers
 
 # =============================================================================
 attr_dict = get_schema("external_url", SCHEMA_FN_PATHS)
@@ -25,6 +26,15 @@ class ExternalUrl(Base):
 
     def __init__(self, **kwargs):
 
-        self.description = None
-        self.url = None
         super().__init__(attr_dict=attr_dict, **kwargs)
+
+    def read_dict(self, input_dict):
+        """
+
+        :param input_dict: DESCRIPTION
+        :type input_dict: TYPE
+        :return: DESCRIPTION
+        :rtype: TYPE
+
+        """
+        helpers._read_element(self, input_dict, "external_url")
