@@ -11,7 +11,9 @@ Created on Wed Dec 23 21:30:36 2020
 # =============================================================================
 # Imports
 # =============================================================================
-from mt_metadata.base.helpers import write_lines
+from xml.etree import cElementTree as et
+
+from mt_metadata.base.helpers import write_lines, element_to_string
 from mt_metadata.base import get_schema, Base
 from .standards import SCHEMA_FN_PATHS
 
@@ -23,10 +25,18 @@ class Magnetometer(Base):
 
     def __init__(self, **kwargs):
 
-        self.id = None
-        self.manufacturer = None
-        self.type = None
-        self.model = None
-        self.name = None
-        self.settings = None
         super().__init__(attr_dict=attr_dict, **kwargs)
+
+    def to_xml(self, string=False, required=False):
+        """ """
+
+        root = et.Element(self.__class__.__name__, {"type": self.type})
+
+        for key in ["manufacturer", "name", "id", "settings"]:
+            value = getattr(self, key)
+            if key not in [None, "", "null"]:
+                et.SubElement(root, key).text = value
+
+        if string:
+            return element_to_string(root)
+        return root
