@@ -14,6 +14,7 @@ Created on Wed Dec 23 21:30:36 2020
 from mt_metadata.base.helpers import write_lines
 from mt_metadata.base import get_schema, Base
 from .standards import SCHEMA_FN_PATHS
+from mt_metadata.transfer_functions.io.emtfxml.metadata import helpers
 
 # =============================================================================
 attr_dict = get_schema("primary_data", SCHEMA_FN_PATHS)
@@ -25,5 +26,7 @@ class PrimaryData(Base):
 
     def __init__(self, **kwargs):
 
-        self.filename = None
         super().__init__(attr_dict=attr_dict, **kwargs)
+
+    def read_dict(self, input_dict):
+        helpers._read_element(self, input_dict, "primary_data")
