@@ -23,8 +23,6 @@ class Comment(Base):
     __doc__ = write_lines(attr_dict)
 
     def __init__(self, **kwargs):
-        self.comment = None
-        self.author = None
         self._dt = MTime()
         super().__init__(attr_dict=attr_dict, **kwargs)
 
@@ -34,4 +32,4 @@ class Comment(Base):
 
     @date.setter
     def date(self, dt_str):
-        self._dt.from_str(dt_str)
+        self._dt.parse(dt_str)
