@@ -2,7 +2,7 @@
 """
 Created on Thu Jun 17 15:32:48 2021
 
-:copyright: 
+:copyright:
     Jared Peacock (jpeacock@usgs.gov)
 
 :license: MIT
@@ -14,10 +14,16 @@ from pathlib import Path
 
 
 def get_standard_dirs(dir_path):
-    existing_dirs = list(set([fn.parent.as_posix() for fn in dir_path.rglob("*.json")]))
-    existing_dirs = [fn[fn.rfind("mt_metadata") :] for fn in existing_dirs]
+    existing_dirs = list(
+        set([fn.parent.as_posix() for fn in dir_path.rglob("*.json")])
+    )
+    existing_dirs = [
+        fn[fn.rfind("mt_metadata") :]
+        for fn in existing_dirs
+        if ".git" not in fn
+    ]
 
-    return existing_dirs
+    return sorted(set(existing_dirs))
 
 
 def read_manifest(fn):
@@ -27,7 +33,7 @@ def read_manifest(fn):
         if "recursive-include" in line and "json" in line:
             standards_dir_list.append(line.split()[1])
 
-    return standards_dir_list
+    return sorted(set(standards_dir_list))
 
 
 class TestStandards(unittest.TestCase):
@@ -39,8 +45,9 @@ class TestStandards(unittest.TestCase):
             self.manifest_fn = self.home.parent.joinpath("MANIFEST.in")
 
     def test_standards_exist(self):
-        existing_dirs = sorted(get_standard_dirs(self.home))
-        manifest_dirs = sorted(read_manifest(self.manifest_fn))
+        package_folder = self.home.joinpath(self.home.name)
+        existing_dirs = get_standard_dirs(package_folder)
+        manifest_dirs = read_manifest(self.manifest_fn)
         self.assertListEqual(existing_dirs, manifest_dirs)
 
 
